<?php
/**
 * The main controller that registers callbacks and the main entry point for Amazon.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links\Amazon;

use Tasty\Framework\Abstracts\Subscriber;
use Tasty\Framework\Utils\Vars;
use Tasty_Links\Amazon\Requests\Get_Items;
use Tasty_Links\Admin;
use Tasty_Links\Content_Model;
use WP_Query;

/**
 * Class Amazon Controller
 */
class Controller extends Subscriber {

	/**
	 * The main entry point for Amazon.
	 */
	protected function add_hooks() {
		add_action( 'init', array( $this, 'schedule_refresh_cron' ) );
		add_action( 'tasty_links_refresh_amazon_images', array( $this, 'refresh_amazon_images' ) );
		add_action( 'restrict_manage_posts', array( $this, 'add_links_status_filter' ) );
		add_action( 'pre_get_posts', array( $this, 'apply_amazon_status_filter' ) );
		add_action( 'tasty_links_amazon_get_items_response', array( $this, 'detect_amazon_products_errors' ), 10, 3 );
		add_action( 'tasty_admin_notices', array( $this, 'show_invalid_links_admin_notices' ) );
		add_action( 'tasty_links_after_saving_link', array( $this, 'process_amazon_details_on_url_change' ), 10, 2 );

		add_filter( 'tasty_framework_notification_icon', array( $this, 'maybe_add_notification_menu_icon' ) );
	}

	/**
	 * Schedules the refresh images cron.
	 */
	public function schedule_refresh_cron() {
		$configuration = Configuration::instance();

		if ( wp_next_scheduled( 'tasty_links_refresh_amazon_images' ) ) {
			if ( ! $configuration->has_api_keys() ) {
				wp_unschedule_event( time(), 'tasty_links_refresh_amazon_images' );
			}
			return;
		}

		if ( ! $configuration->has_api_keys() ) {
			return;
		}

		wp_schedule_event( time(), 'hourly', 'tasty_links_refresh_amazon_images' );
	}

	/**
	 * Refreshes the amazon images.
	 */
	public function refresh_amazon_images() {
		// Gets only up to 10 amazon valid tasty link posts.
		$configuration = Configuration::instance();
		if ( ! $configuration->has_api_keys() ) {
			return;
		}

		delete_site_transient( 'tasty-links-amazon-invalid-items' );

		$expired_products = $this->get_expired_products();
		if ( empty( $expired_products ) ) {
			return;
		}

		$this->process_expired_products( $expired_products );
	}

	/**
	 * Gets the expired amazon link ids.
	 *
	 * @return array
	 */
	private function get_expired_amazon_link_ids() {
		$cached_value = Cache::instance()->get_ids();
		if ( is_array( $cached_value ) && ! empty( $cached_value ) ) {
			return (array) $cached_value;
		}

		$time_interval = $this->get_time_interval();

		// Todo: I believe using raw query will be better as long as we cache the data using cache.
		$query = new WP_Query(
			array(
				// phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				'meta_query'     => array(
					'relation' => 'AND',
					array(
						'key'     => Admin::LINK_KEY,
						'value'   => '(?:amazon|amzn)\.',
						'compare' => 'REGEXP',
					),
					array(
						'relation' => 'OR',
						array(
							'key'     => Admin::AMAZON_EXPIRY_META_KEY,
							'compare' => 'NOT EXISTS',
						),
						array(
							'key'     => Admin::AMAZON_EXPIRY_META_KEY,
							'value'   => time() + $time_interval,
							'compare' => '<=',
							'type'    => 'NUMERIC',
						),
					),
					array(
						'key'     => Admin::AMAZON_INVALID_META_KEY,
						'compare' => 'NOT EXISTS',
					),
				),
				'post_type'      => Content_Model::POST_TYPE,
				'posts_per_page' => 100 * 5,
				'fields'         => 'ids',
				'no_found_rows'  => true,
			)
		);

		$ids = $query->have_posts() ? $query->posts : array();
		Cache::instance()->set_ids( $ids );
		wp_reset_postdata();

		return $ids;
	}

	/**
	 * Gets the time interval.
	 *
	 * @since 1.3
	 *
	 * @return int
	 */
	private function get_time_interval() {
		/**
		 * The time interval to check for expired products.
		 *
		 * @since 1.3
		 *
		 * @param int $time_interval The time interval.
		 * @return int
		 */
		return (int) apply_filters( 'tasty_links_amazon_expiry_interval', 2 * HOUR_IN_SECONDS );
	}

	/**
	 * Gets the expired products.
	 *
	 * @return array
	 */
	private function get_expired_products() {
		$ids   = $this->get_expired_amazon_link_ids();
		$links = array();

		foreach ( $ids as $id ) {
			$link  = get_post_meta( $id, Admin::LINK_KEY, true );
			$store = Link_Parser::get_product_store( $link );
			if ( empty( $store ) ) {
				continue;
			}

			$asin = get_post_meta( $id, Admin::ASIN_KEY, true );
			if ( empty( $asin ) ) {
				$asin = Link_Parser::get_asin_from_link( $link );
				update_post_meta( $id, Admin::ASIN_KEY, $asin );
			}

			if ( empty( $asin ) ) {
				continue;
			}

			if ( ! isset( $links[ $store ] ) ) {
				$links[ $store ] = array();
			}

			$links[ $store ][ $asin ] = $id;
		}

		return $links;
	}

	/**
	 * Processes the expired products.
	 *
	 * @param array $expired_products The expired products.
	 */
	private function process_expired_products( $expired_products ) {
		$get_items = Get_Items::get_instance();
		if ( is_null( $get_items ) ) {
			return;
		}

		$stores_count_to_process   = (int) apply_filters( 'tasty_links_amazon_batch_stores_count', 1, $expired_products );
		$products_count_to_process = (int) apply_filters( 'tasty_links_amazon_batch_products_count', 10, $expired_products );

		$expired_products = array_slice( $expired_products, 0, $stores_count_to_process, true );

		$responses   = array();
		$product_map = array();
		foreach ( $expired_products as $store => $products ) {
			// Get only 10 products.
			$sliced_products = array_slice( $products, 0, $products_count_to_process, true );
			$api_response    = $get_items->get_product_details( array_keys( $sliced_products ), $store );
			if ( ! empty( $api_response['error'] ) ) {
				continue;
			}

			$responses   = array_merge( $responses, $api_response );
			$product_map = array_merge( $product_map, $sliced_products );
		}

		foreach ( $responses as $asin => $product ) {
			if ( ! isset( $product_map[ $asin ] ) ) {
				continue;
			}
			$this->update_product_details( $product_map[ $asin ], $product );
		}

		Cache::instance()->remove_ids( array_values( $product_map ) );
	}

	/**
	 * Process amazon details for one tasty link item by post ID.
	 *
	 * @param int $id Tasty link item Post ID.
	 *
	 * @return void
	 */
	private function process_one_product( $id ) {
		$get_items = Get_Items::get_instance();
		if ( is_null( $get_items ) ) {
			return;
		}

		$link = get_post_meta( $id, Admin::LINK_KEY, true );
		if ( empty( $link ) ) {
			return;
		}

		$asin = get_post_meta( $id, Admin::ASIN_KEY, true );
		if ( empty( $asin ) ) {
			$asin = Link_Parser::get_asin_from_link( $link );
			update_post_meta( $id, Admin::ASIN_KEY, $asin );
		}

		if ( empty( $asin ) ) {
			return;
		}

		$store = Link_Parser::get_product_store( $link );
		if ( empty( $store ) ) {
			return;
		}

		$api_response = $get_items->get_product_details( array( $asin ), $store );
		if ( ! empty( $api_response['error'] ) ) {
			return;
		}

		$product = array_shift( $api_response );
		$this->update_product_details( $id, $product );
	}

	/**
	 * Updates the product details.
	 *
	 * @param int   $id      The post id.
	 * @param array $product The product details.
	 */
	private function update_product_details( $id, $product ) {
		$image_url = $product['image'];

		update_post_meta( $id, Admin::IMAGE_URL_META_KEY, $image_url );
		update_post_meta( $id, Admin::AMAZON_EXPIRY_META_KEY, strtotime( '+24 hours' ) );

		if ( ! empty( $product['width'] ) ) {
			update_post_meta( $id, Admin::IMAGE_URL_WIDTH_META_KEY, absint( $product['width'] ) );
		}

		if ( ! empty( $product['height'] ) ) {
			update_post_meta( $id, Admin::IMAGE_URL_HEIGHT_META_KEY, absint( $product['height'] ) );
		}
	}

	/**
	 * Get details of invalid amazon links.
	 *
	 * @return array
	 */
	private function get_invalid_amazon_links_details() {
		$cached = get_site_transient( 'tasty-links-amazon-invalid-items' );
		if ( false !== $cached ) {
			return $cached;
		}

		$query  = new WP_Query(
			array(
				// phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				'meta_query'     => array(
					array(
						'key'     => Admin::AMAZON_INVALID_META_KEY,
						'compare' => 'EXISTS',
					),
				),
				'post_type'      => Content_Model::POST_TYPE,
				'posts_per_page' => -1,
				'fields'         => 'ids',
				'no_found_rows'  => true,
			)
		);
		$output = array(
			$query->post_count,
			$query->posts,
		);

		set_site_transient( 'tasty-links-amazon-invalid-items', $output, 15 * MINUTE_IN_SECONDS );

		return $output;
	}

	/**
	 * Check if we can show amazon link status filter
	 *
	 * @return bool
	 */
	private function can_show_amazon_status_filter() {
		if ( ! is_admin() ) {
			return false;
		}

		global $post_type;
		if ( Content_Model::POST_TYPE !== $post_type || 'edit-tasty_link' !== get_current_screen()->id ) {
			return false;
		}

		return Configuration::instance()->has_api_keys();
	}

	/**
	 * Add amazon links status filter UI.
	 *
	 * @return void
	 */
	public function add_links_status_filter() {
		if ( ! $this->can_show_amazon_status_filter() ) {
			return;
		}

		list( $count ) = $this->get_invalid_amazon_links_details();
		if ( 0 === $count ) {
			return;
		}

		$data = array(
			'invalid_count'              => $count,
			'tasty_filter_amazon_status' => Vars::get_param( 'tasty-filter-amazon-status' ),
		);
		Admin::show_template_part( 'filter-amazon-status', $data );
	}

	/**
	 * Apply amazon status filter on results.
	 *
	 * @param WP_Query $query Current query instance.
	 *
	 * @return void
	 */
	public function apply_amazon_status_filter( $query ) {
		if ( ! $this->can_show_amazon_status_filter() ) {
			return;
		}

		$tasty_filter_amazon_status = Vars::get_param( 'tasty-filter-amazon-status' );
		if ( 'invalid' !== $tasty_filter_amazon_status ) {
			return;
		}

		// phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
		$query->query_vars['meta_query'] = array(
			array(
				'key'     => Admin::AMAZON_INVALID_META_KEY,
				'compare' => 'EXISTS',
			),
		);
	}

	/**
	 * Detect amazon errors from comparing the request ASINs with resonse ASINs and also the error messages.
	 *
	 * @param array  $request_asins Request ASINs.
	 * @param array  $response_asins Response ASINs.
	 * @param object $response Response object.
	 *
	 * @return void
	 */
	public function detect_amazon_products_errors( $request_asins, $response_asins, $response ) {
		$notvalid_asins = array_diff( $request_asins, $response_asins );

		// Process not valid ASINs.
		$processor = new Notvalid_Processor();

		if ( ! empty( $notvalid_asins ) && ! empty( $response_asins ) ) {
			// If there are some products' images returned and other images have problems.
			$processor->set_asins( $notvalid_asins );
		}

		if ( ! empty( $response->errors ) ) {
			$processor->set_errors( $response->errors );
		}

		$processor->process();
	}

	/**
	 * Show admin notices for invalid amazon links.
	 *
	 * @return void
	 */
	public function show_invalid_links_admin_notices() {
		if ( ! Configuration::instance()->has_api_keys() ) {
			return;
		}

		$this->show_invalid_links_counter_admin_notice();
		$this->show_post_invalid_link_admin_notice();
	}

	/**
	 * Show invalid links counter admin notices.
	 *
	 * @return void
	 */
	private function show_invalid_links_counter_admin_notice() {
		if ( ! in_array( get_current_screen()->id, array( 'toplevel_page_tasty', 'wp-tasty_page_tasty-links', 'edit-tasty_link' ), true ) ) {
			return;
		}

		list( $invalid_links_count, $invalid_links_ids ) = $this->get_invalid_amazon_links_details();
		if ( 0 === $invalid_links_count ) {
			return;
		}

		$data = array(
			'not_valid_count' => $invalid_links_count,
			'not_valid_url'   =>
				( 1 === $invalid_links_count ) ? get_edit_post_link( $invalid_links_ids[0] ) : add_query_arg(
					array(
						'post_type'                  => Content_Model::POST_TYPE,
						'tasty-filter-amazon-status' => 'invalid',
					),
					admin_url( 'edit.php' )
				),
		);

		Admin::show_template_part( 'amazon-errors-admin-notice', $data );
	}

	/**
	 * Can show admin notice in tasty link edit page when it's corrupted amazon link.
	 *
	 * @return bool
	 */
	private function can_show_post_invalid_link_admin_notice() {
		if ( 'tasty_link' !== get_current_screen()->id ) {
			return false;
		}

		global $post_id;
		return ! empty( get_post_meta( $post_id, Admin::AMAZON_INVALID_META_KEY, true ) );
	}

	/**
	 *
	 * Show admin notice in tasty link edit page when it's corrupted amazon link.
	 *
	 * @return void
	 */
	private function show_post_invalid_link_admin_notice() {
		if ( ! $this->can_show_post_invalid_link_admin_notice() ) {
			return;
		}

		Admin::show_template_part( 'amazon-post-error-admin-notice' );
	}

	/**
	 * Show red notification icon beside tasty main admin menu when there are invalid amazon links.
	 *
	 * @param bool $show_icon Show notification icon.
	 *
	 * @return bool
	 */
	public function maybe_add_notification_menu_icon( $show_icon ) {
		if ( $show_icon ) {
			return $show_icon;
		}

		list( $count ) = $this->get_invalid_amazon_links_details();
		return 0 < $count;
	}

	/**
	 * Get product details only when the url changes.
	 *
	 * @param int   $post_id Current tasty link post ID.
	 * @param array $updated_metas Updated metas.
	 *
	 * @return void
	 */
	public function process_amazon_details_on_url_change( $post_id, $updated_metas ) {
		if ( ! in_array( Admin::LINK_KEY, $updated_metas, true ) ) {
			return;
		}

		if ( ! Configuration::instance()->has_api_keys() ) {
			return;
		}

		delete_post_meta( $post_id, Admin::AMAZON_INVALID_META_KEY );
		delete_post_meta( $post_id, Admin::ASIN_KEY );
		delete_site_transient( 'tasty-links-amazon-invalid-items' );

		$this->process_one_product( $post_id );
	}
}
