<?php
/**
 * Class to parse amazon urls and extract data from there.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links\Amazon;

/**
 * Class Link Parser
 */
class Link_Parser {

	/**
	 * Converts the html to a link with image details.
	 *
	 * @param string $html The html pasted in Amazon modal.
	 *
	 * @return array
	 */
	public static function convert_link_to_details( $html ) {
		$data = array();
		$link = self::prepare_link( $html );

		if ( empty( $link ) ) {
			return $data;
		}

		$data['link'] = $link;

		$asin = self::get_asin_from_link( $link );
		if ( empty( $asin ) ) {
			return $data;
		}

		$data['asin'] = $asin;

		self::add_single_product_details( $link, $data );

		return $data;
	}

	/**
	 * Get the product ASIN from the link.
	 *
	 * @param string $link The link.
	 *
	 * @return string
	 */
	public static function get_asin_from_link( $link ) {
		$link = self::get_full_link( $link );
		if ( ! preg_match( '~http[s]?:\/\/.+(?<code>\/gp|\/dp).+(?<asin>[a-zA-Z0-9]{10})~iU', $link, $matches ) ) {
			return '';
		}

		return $matches['asin'];
	}

	/**
	 * Get the full link from the short link.
	 *
	 * @param string $link The link.
	 *
	 * @return string
	 */
	private static function get_full_link( $link ) {
		if ( ! str_contains( $link, 'amzn.to' ) ) {
			return $link;
		}

		$short_request = wp_remote_get( $link, array( 'redirection' => 0 ) );
		if ( is_wp_error( $short_request ) || empty( $short_request['headers'] ) || empty( $short_request['headers']['location'] ) ) {
			return '';
		}

		return $short_request['headers']['location'];
	}

	/**
	 * Get the product ASIN from the link.
	 *
	 * @param string $html_or_link The link.
	 *
	 * @return string
	 */
	public static function prepare_link( $html_or_link ) {
		// Check if it's a link without html.
		$is_link = preg_match( '~http[s]?:\/\/~iU', $html_or_link );
		if ( $is_link ) {
			return $html_or_link;
		}

		if ( ! preg_match( '~(?:<a[^>]+href=["\'])*(?<link>http[^"\']+)["\'][^>]*~iU', $html_or_link, $matches ) ) {
			return '';
		}
		return $matches['link'];
	}

	/**
	 * Get the product store from the link.
	 *
	 * @param string $link The link.
	 *
	 * @return string
	 */
	public static function get_product_store( $link ) {
		$link = self::get_full_link( $link );
		if ( ! preg_match( '~' . Locales::get_stores_regex() . '~iU', $link, $matches ) ) {
			return '';
		}
		return $matches['store'];
	}

	/**
	 * Add product details to the data.
	 *
	 * @since 1.3
	 *
	 * @param string $link The link.
	 * @param array  $data The data.
	 * @return void
	 */
	private static function add_single_product_details( $link, &$data ) {
		$get_items = Requests\Get_Items::get_instance();
		if ( is_null( $get_items ) ) {
			$data['error'] = sprintf(
				/* translators: %1$s - opening a link, %2$s - closing a link */
				__( 'Please %1$sadd Amazon PA API keys%2$s to get images and keep them updated per Amazon terms.', 'tasty-links' ),
				'<a href="' . esc_url( admin_url( 'admin.php?page=tasty-links&tab=settings' ) ) . '">',
				'</a>'
			);
			return;
		}

		$asin             = $data['asin'];
		$store            = self::get_product_store( $link );
		$product_response = $get_items->get_product_details( $asin, $store );

		if ( ! empty( $product_response['error'] ) ) {
			$data['error'] = $product_response['error'];
		}

		if ( empty( $product_response[ $asin ] ) ) {
			return;
		}

		// Get the info for the single product.
		$product_response = $product_response[ $asin ];

		if ( ! empty( $product_response['image'] ) ) {
			$data['image'] = $product_response['image'];
			$data['title'] = $product_response['title'];
		}
	}
}
