<?php
/**
 * Amazon Not valid product processor.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links\Amazon;

use Tasty_Links\Admin;
use Tasty_Links\Content_Model;
use WP_Query;

/**
 * Amazon Not valid product processor.
 */
class Notvalid_Processor {

	/**
	 * Invalid ASINs to process.
	 *
	 * @var array
	 */
	private $asins = array();

	/**
	 * Invalid errors array of objects.
	 *
	 * @var array
	 */
	private $errors = array();

	/**
	 * Amazon errors codes/messages mapping.
	 *
	 * @var array
	 */
	private $amazon_errors = array();

	/**
	 * Class constructor.
	 * Load default amazon error mappings.
	 */
	public function __construct() {
		$notvalid_api_credentials_msg = __( 'Please verify your Amazon API Key, Secret and Store ID are entered correctly.', 'tasty-links' );
		$this->amazon_errors          = array(
			'IncompleteSignature' => $notvalid_api_credentials_msg,
			'InvalidSignature'    => $notvalid_api_credentials_msg,
			'UnrecognizedClient'  => $notvalid_api_credentials_msg,
		);
	}

	/**
	 * Set invalid ASINs.
	 *
	 * @param array $notvalid_asins Not valid ASINs.
	 *
	 * @return void
	 */
	public function set_asins( $notvalid_asins ) {
		$this->asins = $notvalid_asins;
	}

	/**
	 * Set errors from amazon response.
	 *
	 * @param array $errors Amazon reponse errors.
	 *
	 * @return void
	 */
	public function set_errors( $errors ) {
		$this->errors = $errors;
	}

	/**
	 * Start processing.
	 *
	 * @return void
	 */
	public function process() {
		$this->process_errors();
		$this->process_invalid_asins();
	}

	/**
	 * Process invalid ASINs.
	 *
	 * @return void
	 */
	private function process_invalid_asins() {
		if ( empty( $this->asins ) ) {
			return;
		}

		$this->asins = array_unique( $this->asins );

		$ids = $this->get_product_ids();
		if ( empty( $ids ) ) {
			return;
		}

		$this->remove_ids_from_cache( $ids );

		foreach ( $ids as $id ) {
			$this->mark_link_as_invalid( $id );
		}

		$this->asins = array();// Reset.
	}

	/**
	 * Process amazon response errors.
	 *
	 * @return void
	 */
	private function process_errors() {
		if ( empty( $this->errors ) ) {
			delete_site_transient( 'tasty-links-amazon-last-errors' );
			return;
		}

		$errors = array();

		foreach ( $this->errors as $error ) {
			$errors[ $error->Code ] = isset( $this->amazon_errors[ $error->Code ] ) ? $this->amazon_errors[ $error->Code ] : $error->Message;
			$this->parse_error_message( $error->Message );
		}

		set_site_transient( 'tasty-links-amazon-last-errors', $errors, HOUR_IN_SECONDS );
	}

	/**
	 * Process error message to extract ASIN when it's there.
	 *
	 * @param string $error_message Error message to parse.
	 *
	 * @return void
	 */
	private function parse_error_message( $error_message ) {
		if ( ! preg_match( '~^The ItemId (?<asin>[a-zA-Z0-9]{10}) ~i', $error_message, $matches ) ) {
			return;
		}
		$this->asins[] = $matches['asin'];
	}

	/**
	 * Get product IDs for products ASINs.
	 *
	 * @return array|int[]
	 */
	private function get_product_ids() {
		$query = new WP_Query(
			array(
				// phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				'meta_query'     => array(
					array(
						'key'     => Admin::ASIN_KEY,
						'value'   => $this->asins,
						'compare' => 'IN',
					),
				),
				'post_type'      => Content_Model::POST_TYPE,
				'posts_per_page' => 5 * 100,
				'fields'         => 'ids',
			)
		);

		return $query->have_posts() ? $query->posts : array();
	}

	/**
	 * Remove invalid product IDs from expired cache.
	 *
	 * @param array $ids Invalid product IDs to remove.
	 *
	 * @return void
	 */
	private function remove_ids_from_cache( $ids ) {
		Cache::instance()->remove_ids( $ids );
	}

	/**
	 * Invalidate the provided link by ID.
	 *
	 * @param int $id Tasty link post ID to invalidate.
	 *
	 * @return void
	 */
	private function mark_link_as_invalid( $id ) {
		update_post_meta( $id, Admin::AMAZON_INVALID_META_KEY, 1 );
	}
}
