<?php
/**
 * Amazon API Signature class.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links\Amazon\Requests;

use Tasty_Links\Amazon\Configuration;

/**
 * Amazon Base API request class.
 */
abstract class Base_Request {

	/**
	 * Region name.
	 *
	 * @var string
	 */
	private $region_name = 'us-east-1';

	/**
	 * Service name.
	 *
	 * @var string
	 */
	private $service_name = 'ProductAdvertisingAPI';

	/**
	 * HTTP method type.
	 *
	 * @var string
	 */
	private $http_method_name = 'POST';

	/**
	 * AWS headers.
	 *
	 * @var array
	 */
	private $aws_headers = array();

	/**
	 * HMAC algorithm.
	 *
	 * @var string
	 */
	private $hmac_algorithm = 'AWS4-HMAC-SHA256';

	/**
	 * AWS request type.
	 *
	 * @var string
	 */
	private $aws4_request = 'aws4_request';

	/**
	 * Signed header.
	 *
	 * @var null|string
	 */
	private $str_signed_header = null;

	/**
	 * X-amz-date.
	 *
	 * @var null|string
	 */
	private $xamz_date = null;

	/**
	 * Current date.
	 *
	 * @var null|string
	 */
	private $current_date = null;

	/**
	 * Host.
	 *
	 * @var string
	 */
	private $host = 'webservices.amazon.com';

	/**
	 * Configuration.
	 *
	 * @var Configuration
	 */
	protected $configuration;

	/**
	 * Get the instance.
	 *
	 * @since 1.3
	 *
	 * @return static|null
	 */
	public static function get_instance() {
		static $instance;

		if ( ! $instance ) {
			$configuration = Configuration::instance();
			if ( ! $configuration->has_api_keys() ) {
				return null;
			}
			$instance = new static( $configuration );
		}

		return $instance;
	}

	/**
	 * Base_Request constructor.
	 *
	 * @param Configuration $configuration The configuration.
	 */
	public function __construct( $configuration ) {
		$this->configuration = $configuration;
		$this->xamz_date     = $this->get_time_stamp();
		$this->current_date  = $this->get_date();
	}

	/**
	 * Set the region name.
	 *
	 * @param string $region_name The region name.
	 */
	protected function set_region_name( $region_name ) {
		$this->region_name = $region_name;
	}

	/**
	 * Set the host.
	 *
	 * @param string $host The host.
	 */
	protected function set_host( $host ) {
		$this->host = 'webservices.' . $host;
	}

	/**
	 * Get the host.
	 *
	 * @return string
	 */
	protected function get_host() {
		return $this->host;
	}

	/**
	 * Add a header.
	 *
	 * @param string $header_name  The header name.
	 * @param string $header_value The header value.
	 */
	protected function add_header( $header_name, $header_value ) {
		$this->aws_headers[ $header_name ] = $header_value;
	}

	/**
	 * Prepare the canonical request.
	 *
	 * @return string
	 */
	private function prepare_canonical_request() {
		$canonical_url  = $this->http_method_name . "\n";
		$canonical_url .= $this->get_path() . "\n\n";
		$signed_headers = '';
		foreach ( $this->aws_headers as $key => $value ) {
			$signed_headers .= $key . ';';
			$canonical_url  .= $key . ':' . $value . "\n";
		}
		$canonical_url .= "\n";

		$this->str_signed_header = substr( $signed_headers, 0, - 1 );

		$canonical_url .= $this->str_signed_header . "\n";
		$canonical_url .= $this->generate_hex( $this->get_payload() );
		return $canonical_url;
	}

	/**
	 * Prepare the string to sign.
	 *
	 * @param string $canonical_url The canonical url.
	 *
	 * @return string
	 */
	private function prepare_string_to_sign( $canonical_url ) {
		$string_to_sign  = $this->hmac_algorithm . "\n";
		$string_to_sign .= $this->xamz_date . "\n";
		$string_to_sign .= $this->current_date . '/' . $this->region_name . '/';
		$string_to_sign .= $this->service_name . '/' . $this->aws4_request . "\n";
		$string_to_sign .= $this->generate_hex( $canonical_url );
		return $string_to_sign;
	}

	/**
	 * Calculate the signature.
	 *
	 * @param string $string_to_sign The string to sign.
	 *
	 * @return string
	 */
	private function calculate_signature( $string_to_sign ) {
		$signature_key = $this->get_signature_key( $this->configuration->api_secret, $this->current_date, $this->region_name, $this->service_name );

		$signature         = hash_hmac( 'sha256', $string_to_sign, $signature_key, true );
		$str_hex_signature = strtolower( bin2hex( $signature ) );
		return $str_hex_signature;
	}

	/**
	 * Get the headers.
	 *
	 * @return array
	 */
	protected function get_headers() {
		$this->add_header( 'x-amz-date', $this->xamz_date );
		$this->add_header( 'content-encoding', 'amz-1.0' );
		$this->add_header( 'content-type', 'application/json; charset=utf-8' );
		$this->add_header( 'host', $this->get_host() );
		$this->add_header( 'x-amz-target', $this->get_target() );
		ksort( $this->aws_headers );

		$canonical_url  = $this->prepare_canonical_request();
		$string_to_sign = $this->prepare_string_to_sign( $canonical_url );
		$signature      = $this->calculate_signature( $string_to_sign );
		if ( $signature ) {
			$this->add_header( 'Authorization', $this->build_authorization_string( $signature ) );
			return $this->aws_headers;
		}

		return array();
	}

	/**
	 * Build the authorization string.
	 *
	 * @param string $str_signature The signature.
	 *
	 * @return string
	 */
	private function build_authorization_string( $str_signature ) {
		return $this->hmac_algorithm . ' Credential=' . $this->configuration->api_key . '/' .
			$this->get_date() . '/' . $this->region_name . '/' . $this->service_name . '/' .
			$this->aws4_request . ',SignedHeaders=' . $this->str_signed_header .
			',Signature=' . $str_signature;
	}

	/**
	 * Generate the hex.
	 *
	 * @param string $data The data.
	 *
	 * @return string
	 */
	private function generate_hex( $data ) {
		return strtolower( bin2hex( hash( 'sha256', $data, true ) ) );
	}

	/**
	 * Get the signature key.
	 *
	 * @param string $key          The key.
	 * @param string $date         The date.
	 * @param string $region_name  The region name.
	 * @param string $service_name The service name.
	 *
	 * @return string
	 */
	private function get_signature_key( $key, $date, $region_name, $service_name ) {
		$ksecret  = 'AWS4' . $key;
		$kdate    = hash_hmac( 'sha256', $date, $ksecret, true );
		$kregion  = hash_hmac( 'sha256', $region_name, $kdate, true );
		$kservice = hash_hmac( 'sha256', $service_name, $kregion, true );
		$ksigning = hash_hmac( 'sha256', $this->aws4_request, $kservice, true );

		return $ksigning;
	}

	/**
	 * Get the time stamp.
	 *
	 * @return string
	 */
	private function get_time_stamp() {
		return gmdate( 'Ymd\THis\Z' );
	}

	/**
	 * Get the current date.
	 *
	 * @return string
	 */
	private function get_date() {
		return gmdate( 'Ymd' );
	}

	/**
	 * Get the url path.
	 *
	 * @return string
	 */
	abstract protected function get_path();

	/**
	 * Prepare the payload.
	 *
	 * @return string
	 */
	abstract protected function get_payload();

	/**
	 * Get the target url.
	 *
	 * @return string
	 */
	abstract protected function get_target();
}
