<?php
/**
 * Amazon Get_Items Request.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links\Amazon\Requests;

use Tasty_Links\Amazon\Locales;

/**
 * Amazon Get_Items Request.
 */
class Get_Items extends Base_Request {

	/**
	 * Item IDs.
	 *
	 * @var array
	 */
	private $item_ids = array();

	/**
	 * Marketplace.
	 *
	 * @var string
	 */
	private $marketplace;

	/**
	 * Resources.
	 *
	 * @var array
	 */
	private $resources = array();

	/**
	 * Set the item IDs.
	 *
	 * @param array $item_ids Array of item IDs.
	 */
	private function set_item_ids( $item_ids ) {
		$this->item_ids = array_map(
			function ( $item ) {
				return (string) $item;
			},
			(array) $item_ids
		);
	}

	/**
	 * Get the item IDs.
	 *
	 * @return array
	 */
	public function get_item_ids() {
		return $this->item_ids;
	}

	/**
	 * Set the marketplace.
	 *
	 * @param string $marketplace Marketplace.
	 */
	private function set_marketplace( $marketplace ) {
		$this->marketplace = 'www.' . $marketplace;
	}

	/**
	 * Get the marketplace.
	 *
	 * @return string
	 */
	public function get_marketplace() {
		return $this->marketplace;
	}

	/**
	 * Get the partner tag.
	 *
	 * @return string
	 */
	private function get_partner_tag() {
		return $this->configuration->tracking_id;
	}

	/**
	 * Get the partner type.
	 *
	 * @return string
	 */
	private function get_partner_type() {
		return 'Associates';
	}

	/**
	 * Set the resources.
	 *
	 * @param array $resources Array of resources.
	 */
	private function set_resources( $resources ) {
		$this->resources = (array) $resources;
	}

	/**
	 * Get the resources.
	 *
	 * @return array
	 */
	private function get_resources() {
		return $this->resources;
	}

	/**
	 * Prepare the payload.
	 *
	 * @return string
	 */
	protected function get_payload() {
		return wp_json_encode(
			array(
				'ItemIds'     => $this->get_item_ids(),
				'Marketplace' => $this->get_marketplace(),
				'PartnerTag'  => $this->get_partner_tag(),
				'PartnerType' => $this->get_partner_type(),
				'Resources'   => $this->get_resources(),
			)
		);
	}

	/**
	 * Get the url path.
	 *
	 * @return string
	 */
	protected function get_path() {
		return '/paapi5/getitems';
	}

	/**
	 * Get the target url.
	 *
	 * @return string
	 */
	protected function get_target() {
		return 'com.amazon.paapi5.v1.ProductAdvertisingAPIv1.GetItems';
	}

	/**
	 * Send the API request.
	 *
	 * @return object
	 */
	public function send_request() {
		$request_args = array(
			'headers' => $this->get_headers(),
			'body'    => $this->get_payload(),
		);

		$response = wp_safe_remote_post( 'https://' . $this->get_host() . $this->get_path(), $request_args );
		return $this->handle_response( $response );
	}

	/**
	 * Check the correct structure and handle error codes.
	 *
	 * @param object $response Response object.
	 *
	 * @return object
	 */
	private function handle_response( $response ) {
		$return          = new \stdClass();
		$return->success = false;
		if ( is_wp_error( $response ) ) {
			$return->error = $response->get_error_message();
			return $return;
		}

		$response = json_decode( wp_remote_retrieve_body( $response ) );
		if ( ! empty( $response->Errors ) ) {
			$return->errors = $response->Errors;
			foreach ( $response->Errors as $error ) {
				$return->error = $error->Code . ' - ' . $error->Message;
			}
			// Only return here and don't continue if we don't have ItemsResult key.
			if ( empty( $response->ItemsResult ) ) {
				return $return;
			}
		}

		$return          = $response;
		$return->success = true;

		return $return;
	}

	/**
	 * Get product details.
	 *
	 * @param array  $asins Array of ASINs.
	 * @param string $store Store name.
	 *
	 * @return array
	 */
	public function get_product_details( $asins, $store ) {
		$locale = Locales::get_one( $store );
		if ( $locale ) {
			$this->set_marketplace( $store );
			$this->set_host( $store );
			$this->set_region_name( $locale['region'] );
		}
		$this->set_item_ids( $asins );
		$this->set_resources(
			array(
				'ItemInfo.Title',
				'Images.Primary.Large',
			)
		);

		$response = $this->send_request();

		do_action( 'tasty_links_amazon_get_items_response', $this->get_item_ids(), $this->get_response_item_ids( $response ), $response );

		if ( ! $response->success ) {
			// Don't continue if there was an error.
			return (array) $response;
		}

		return $this->prepare_products( $response );
	}

	/**
	 * Prepare products from the response.
	 *
	 * @param object $response Response object.
	 *
	 * @return array
	 */
	private function prepare_products( $response ) {
		$products = array();

		foreach ( $response->ItemsResult->Items as $item ) {
			$products[ $item->ASIN ] = array(
				'title'  => $item->ItemInfo->Title->DisplayValue,
				'image'  => $item->Images->Primary->Large->URL,
				'width'  => $item->Images->Primary->Large->Width,
				'height' => $item->Images->Primary->Large->Height,
			);
		}

		return $products;
	}

	/**
	 * Get ASINs from response.
	 *
	 * @param object $response Response object.
	 *
	 * @return array
	 */
	private function get_response_item_ids( $response ) {
		if ( empty( $response->ItemsResult ) || empty( $response->ItemsResult->Items ) ) {
			return array();
		}

		$item_ids = array();
		foreach ( $response->ItemsResult->Items as $item ) {
			$item_ids[] = $item->ASIN;
		}

		return $item_ids;
	}
}
