<?php
/**
 * Manages block editor registration and configuration.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links;

/**
 * Manages block editor registration and configuration.
 */
class Block_Editor {

	/**
	 * Block type name for individual featured link block.
	 *
	 * @var string
	 */
	const INDIVIDUAL_LINK_BLOCK_TYPE = 'wp-tasty/tasty-links-individual';

	/**
	 * Block type name for collection of featured links.
	 *
	 * @var string
	 */
	const FEATURED_LINKS_BLOCK_TYPE = 'wp-tasty/tasty-links-featured';

	/**
	 * Register the scripts and block type.
	 */
	public static function action_init_register() {
		if ( ! function_exists( 'register_block_type' ) ) {
			return;
		}
		register_block_type(
			self::INDIVIDUAL_LINK_BLOCK_TYPE,
			array(
				'attributes'      => array(
					'id' => array(
						'type' => 'number',
					),
				),
				'render_callback' => array( __CLASS__, 'render_individual_link_block' ),
			)
		);
		wp_register_style( 'tasty-links-featured-links-block', plugins_url( 'assets/css/featured-links-block.css', __DIR__ ), null, TASTY_LINKS_PLUGIN_VERSION );
		register_block_type(
			self::FEATURED_LINKS_BLOCK_TYPE,
			array(
				'style' => 'tasty-links-featured-links-block',
			)
		);
	}

	/**
	 * Enqueue our simple stylesheet in the block editor.
	 */
	public static function action_enqueue_block_editor_assets() {
		$fmtime = filemtime( dirname( __DIR__ ) . '/assets/css/editor.css' );
		wp_enqueue_style( 'tasty-pins-editor', plugins_url( 'assets/css/editor.css', __DIR__ ), null, $fmtime );

		$fmtime = 0;
		if ( file_exists( dirname( __DIR__ ) . '/assets/dist/block-editor.build.js' ) ) {
			$fmtime = filemtime( dirname( __DIR__ ) . '/assets/dist/block-editor.build.js' );
		}
		wp_enqueue_script(
			'tasty-links-block-editor',
			plugins_url( 'assets/dist/block-editor.build.js', __DIR__ ),
			array(
				'wp-components',
				'wp-dom',
				'wp-editor',
				'wp-element',
				'wp-i18n',
				'wp-rich-text',
				'wp-url',
			),
			$fmtime,
			true
		);

		$link_options = array(
			array(
				'label' => __( 'No link selected', 'tasty-links' ),
				'value' => 0,
			),
		);
		foreach ( Content_Model::get_links() as $link ) {
			$link_options[] = array(
				'label' => $link->post_title,
				'value' => $link->ID,
			);
		}

		wp_localize_script(
			'tasty-links-block-editor',
			'tastyLinksBlockEditor',
			array(
				'individualLinkBlockType' => self::INDIVIDUAL_LINK_BLOCK_TYPE,
				'featuredLinksBlockType'  => self::FEATURED_LINKS_BLOCK_TYPE,
				'linkOptions'             => $link_options,
			)
		);
		if ( function_exists( 'wp_set_script_translations' ) ) {
			wp_set_script_translations( 'tasty-links-block-editor', 'tasty-links' );
		}
	}

	/**
	 * Renders the single link block.
	 *
	 * @param array $attrs Block attributes.
	 */
	public static function render_individual_link_block( $attrs ) {
		if ( empty( $attrs['id'] ) ) {
			return;
		}
		return Admin::get_template_part(
			'tasty-link-card',
			array(
				'equipment_ids' => array( $attrs['id'] ),
			)
		);
	}

	/**
	 * Runs when blocks are rendered on the frontend.
	 *
	 * @param string $block_output Existing block output.
	 * @param array  $block        Block data.
	 * @return string
	 */
	public static function filter_render_block( $block_output, $block ) {
		if ( self::FEATURED_LINKS_BLOCK_TYPE !== $block['blockName'] ) {
			return $block_output;
		}
		$link_ids = array();
		if ( ! empty( $block['innerBlocks'] ) ) {
			foreach ( $block['innerBlocks'] as $inner_block ) {
				if ( self::INDIVIDUAL_LINK_BLOCK_TYPE !== $inner_block['blockName'] ) {
					continue;
				}
				if ( ! empty( $inner_block['attrs']['id'] ) ) {
					$link_ids[] = $inner_block['attrs']['id'];
				}
			}
		}
		$shareasale = get_option( Admin::SHAREASALE_OPTION, '' );
		if ( $shareasale ) {
			$block_output .= '<div class="tasty-links-plug">';
			$block_output .= esc_html__( 'Featured Links powered by', 'tasty-links' );
			$block_output .= '<a href="' . esc_url( sprintf( 'https://shareasale.com/r.cfm?b=1178987&u=%s&m=69860&urllink=&afftrack=tlattr', $shareasale ) ) . '" target="_blank" rel="nofollow">';
			$block_output .= '<img nopin="nopin" src="' . esc_url( plugins_url( 'assets/images/tasty-links-neutral.svg', __DIR__ ) ) . '" scale="0" height="20px">';
			$block_output .= '</a>';
			$block_output .= '</div>';
		}
		if ( Content_Model::should_disclosure_be_appended_or_prepended( $link_ids, 'featured_links' )
			&& ! has_filter( 'the_content', array( 'Tasty_Links\Frontend', 'filter_the_content_late_for_integration' ), 100 ) ) {
			add_filter( 'the_content', array( 'Tasty_Links\Frontend', 'filter_the_content_late_for_integration' ), 100 );
		}

		$block_content_with_disclosure = preg_replace( '#(</section>)#i', Content_Model::apply_disclosure_featured_links( '', $link_ids ) . '$1', $block_output, 1 );
		if ( is_null( $block_content_with_disclosure ) ) {
			return $block_output;
		}
		return (string) $block_content_with_disclosure;
	}
}
