<?php
/**
 * Defines the plugin content model
 *
 * @package Tasty_Links
 */

namespace Tasty_Links;

/**
 * Defines the plugin content model
 */
class Content_Model {

	/**
	 * Base post type name.
	 *
	 * @var string
	 */
	const POST_TYPE = 'tasty_link';

	/**
	 * Registers the post types.
	 */
	public static function action_init_register_post_types() {

		$args           = array(
			'hierarchical' => false,
			'public'       => false,
			'show_ui'      => true,
			'rewrite'      => false,
			'menu_icon'    => 'dashicons-admin-links',
			'supports'     => array( 'title', 'thumbnail' ),
			'show_in_menu' => false,
			'capabilities' => array(
				'edit_post'          => Admin::CAPABILITY,
				'read_post'          => Admin::CAPABILITY,
				'delete_post'        => Admin::CAPABILITY,
				'edit_posts'         => Admin::CAPABILITY,
				'edit_others_posts'  => Admin::CAPABILITY,
				'create_posts'       => Admin::CAPABILITY,
				'delete_posts'       => Admin::CAPABILITY,
				'publish_posts'      => Admin::CAPABILITY,
				'read_private_posts' => Admin::CAPABILITY,
			),
		);
		$args['labels'] = array(
			'name'               => __( 'Links', 'tasty-links' ),
			'singular_name'      => __( 'Tasty Link', 'tasty-links' ),
			'all_items'          => __( 'All Tasty Links', 'tasty-links' ),
			'new_item'           => __( 'New Tasty Link', 'tasty-links' ),
			'add_new'            => __( 'Add New', 'tasty-links' ),
			'add_new_item'       => __( 'Add New Tasty Link', 'tasty-links' ),
			'edit_item'          => __( 'Edit Tasty Link', 'tasty-links' ),
			'view_item'          => __( 'View Tasty Links', 'tasty-links' ),
			'search_items'       => __( 'Search Tasty Links', 'tasty-links' ),
			'not_found'          => __( 'No links found', 'tasty-links' ),
			'not_found_in_trash' => __( 'No links found in trash', 'tasty-links' ),
			'parent_item_colon'  => __( 'Parent link', 'tasty-links' ),
			'menu_name'          => __( 'Tasty Links', 'tasty-links' ),
		);
		register_post_type( self::POST_TYPE, $args );

		global $wpt_replace_count;
		$wpt_replace_count = array();
	}

	/**
	 * Registers our cron events.
	 */
	public static function action_init_register_cron_events() {
		if ( ! wp_next_scheduled( 'tasty_links_process_height_width' ) ) {
			wp_schedule_event( time(), 'twicedaily', 'tasty_links_process_height_width' );
		}
	}

	/**
	 * Filter the "Enter title here" to change to "Enter keyword here"
	 *
	 * @param string $message Placeholder message.
	 * @return string
	 */
	public static function filter_enter_title_here( $message ) {
		$screen = get_current_screen();
		if ( self::POST_TYPE !== $screen->id ) {
			return $message;
		}
		return __( 'Enter link keyword here', 'tasty-links' );
	}

	/**
	 * Cron callback to generate height/width for images with a manual image_url.
	 */
	public static function action_tasty_links_process_height_width() {
		$query = new \WP_Query(
			array(
				// phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				'meta_query'     => array(
					array(
						'key'     => Admin::IMAGE_URL_META_KEY,
						'value'   => '',
						'compare' => '!=',
					),
				),
				'post_status'    => 'any',
				'post_type'      => self::POST_TYPE,
				'posts_per_page' => -1,
			)
		);
		foreach ( $query->posts as $post ) {
			self::calculate_image_url_height_width( $post->ID );
		}
	}

	/**
	 * Gets all of the Tasty Links.
	 *
	 * @return array
	 */
	public static function get_links() {

		$val = wp_cache_get( 'all-links', 'tasty-links' );
		if ( false !== $val ) {
			return $val;
		}

		$query = new \WP_Query(
			array(
				'posts_per_page'         => -1,
				'post_type'              => self::POST_TYPE,
				'post_status'            => 'publish',
				'update_post_term_cache' => false,
				'update_post_meta_cache' => true,
				'no_found_rows'          => true,
				'orderby'                => 'post_title',
				'order'                  => 'ASC',
			)
		);
		// Non-persistent group.
		wp_cache_set( 'all-links', $query->posts, 'tasty-links' );
		return $query->posts;
	}

	/**
	 * Whether or not the disclosure should be included, depending on the links inserted.
	 *
	 * @param array  $link_ids Set of link ids.
	 * @param string $context  Context in which the function is being called.
	 * @return boolean
	 */
	public static function should_disclosure_be_appended_or_prepended( $link_ids, $context ) {
		/**
		 * Allow the disclosure to be disabled entirely.
		 */
		if ( apply_filters( "tasty_links_disable_{$context}_general_disclosure", false ) ) {
			return false;
		}

		if ( empty( $link_ids ) ) {
			return false;
		}
		$should_be = false;
		foreach ( $link_ids as $link_id ) {
			if ( 'off' !== get_post_meta( $link_id, Admin::APPEND_DISCLOSURE_META_KEY, true ) ) {
				$should_be = true;
			}
		}
		return $should_be;
	}

	/**
	 * Processes Link + image HTML to find the link and the image.
	 *
	 * @param string $html Existing HTML.
	 * @return array
	 */
	public static function get_link_image_from_html( $html ) {
		// Get a url from html image.
		if ( preg_match(
			'#<a[^>]+href=["\']([^"\']+)["\'][^>]*>.*<img[^>]+src=["\']([^"\']+)["\']#U',
			$html,
			$matches
		) ) {
			$data = array(
				'link'  => sanitize_text_field( $matches[1] ),
				'image' => sanitize_text_field( $matches[2] ),
			);

			$asin = Amazon\Link_Parser::get_asin_from_link( $data['link'] );
			if ( $asin ) {
				$data['asin'] = $asin;
			}
			return $data;
		}

		// Get a url from html link.
		if ( preg_match(
			'#<a[^>]+href=["\']([^"\']+)["\'][^>]*>.*#U',
			$html,
			$matches
		) ) {
			$html = $matches[1];
		}

		return Amazon\Link_Parser::convert_link_to_details( $html );
	}

	/**
	 * If the image is missing from the API response,
	 * show an error message and die.
	 *
	 * @since 1.3
	 *
	 * @param array $data Data from the API.
	 * @return string
	 */
	public static function maybe_get_error_message( $data ) {
		if ( ! empty( $data['image'] ) ) {
			return '';
		}

		$message = __( 'Featured image could not be retrieved from Amazon.', 'tasty-links' );
		if ( isset( $data['error'] ) ) {
			$message .= ' ' . $data['error'];
			if ( strpos( $data['error'], 'TooManyRequests' ) !== false ) {
				$message .= ' ' . __( 'This can happen for several reasons: your API keys are less than 72 hours old, you have not had a sale in the last 30 days, or you have exceeded your API rate limit.', 'tasty-links' );
			}
		}
		return $message;
	}

	/**
	 * Calculates the height and width of an image if needed.
	 *
	 * @param integer $post_id ID for the Tasty Link post.
	 */
	public static function calculate_image_url_height_width( $post_id ) {
		$image_url = get_post_meta( $post_id, Admin::IMAGE_URL_META_KEY, true );
		if ( ! $image_url ) {
			return;
		}
		// Already has something.
		if ( get_post_meta( $post_id, Admin::IMAGE_URL_HEIGHT_META_KEY, true )
			|| get_post_meta( $post_id, Admin::IMAGE_URL_WIDTH_META_KEY, true ) ) {
			return;
		}
		if ( 0 === stripos( $image_url, '//' ) ) {
			$image_url = 'https:' . $image_url;
		}
		$size = getimagesize( esc_url_raw( $image_url ) );
		if ( $size ) {
			update_post_meta( $post_id, Admin::IMAGE_URL_WIDTH_META_KEY, absint( $size[0] ) );
			update_post_meta( $post_id, Admin::IMAGE_URL_HEIGHT_META_KEY, absint( $size[1] ) );
		}
	}

	/**
	 * Append the disclosure message to the provided content.
	 *
	 * @param string $content The content to append the disclosure message to.
	 * @param array  $link_ids Provided link IDs.
	 *
	 * @return string
	 */
	public static function apply_disclosure_featured_links( $content, $link_ids = array() ) {
		$disclosure_location = get_option( Admin::DISCLOSURE_LOCATION_OPTION, 'link' );
		if ( 'link' !== $disclosure_location ) {
			return $content;
		}

		if ( ! self::can_append_disclosure_for_links( $link_ids ) ) {
			return $content;
		}

		$disclosure_text = get_option( Admin::DISCLOSURE_TEXT_OPTION, '' );
		if ( empty( $disclosure_text ) ) {
			return $content;
		}

		$disclosure_text = '<div class="tasty-links-general-disclaimer">' . wp_kses_post( wpautop( $disclosure_text ) ) . '</div>';
		return $content . PHP_EOL . $disclosure_text;
	}

	/**
	 * Check if we can append disclosure message for array of links.
	 *
	 * @param array $link_ids Provided Link IDs.
	 *
	 * @return bool
	 */
	private static function can_append_disclosure_for_links( $link_ids = array() ) {
		if ( empty( $link_ids ) ) {
			return false;
		}

		foreach ( $link_ids as $link_id ) {
			if ( 'on' === get_post_meta( $link_id, Admin::APPEND_DISCLOSURE_META_KEY, true ) ) {
				return true;
			}
		}

		return false;
	}
}
