<?php
/**
 * Enhancements to the WordPress frontend.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links;

/**
 * Enhancements to the WordPress frontend.
 */
class Frontend {

	/**
	 * Ignore match string.
	 *
	 * @var string
	 */
	const IGNORE_CONTENT_REGEX = '#<!--\s?\/?tasty-links-ignore\s?-->#i';

	/**
	 * Add any links to the document when present
	 *
	 * @param string $content Post content.
	 * @return string
	 */
	public static function filter_the_content( $content ) {
		global $wp;
		if ( ! is_singular() ) {
			return $content;
		}

		if ( ! tasty_links_should_be_applied( 'frontend' ) ) {
			return $content;
		}

		$post_type = get_post_type( get_the_ID() );
		if ( self::should_disable_auto_linking( $post_type ) ) {
			return $content;
		}

		$skipped_links = array(
			get_permalink( get_the_ID() ),
		);
		if ( ! empty( $wp->request ) ) {
			$skipped_links[] = home_url( $wp->request );
		}

		return self::apply_links_to_content(
			$content,
			'frontend',
			null,
			$skipped_links
		);
	}

	/**
	 * Check if a post type should have auto-linking disabled.
	 *
	 * @since 1.3
	 * @param string $post_type Post type.
	 * @return boolean
	 */
	public static function should_disable_auto_linking( $post_type ) {
		$link_locations = self::get_link_post_types();
		$disable_links  = ! in_array( $post_type, $link_locations, true ) && ! in_array( 'all', $link_locations, true );

		/**
		 * Allows auto-linking to be disabled for a given post type.
		 */
		return apply_filters( "tasty_links_disable_{$post_type}_auto_link", $disable_links );
	}

	/**
	 * Gets the post types that should be auto linked.
	 *
	 * @since 1.3
	 *
	 * @return array
	 */
	private static function get_link_post_types() {
		$default   = array( 'all' );
		$locations = array_filter( (array) get_option( Admin::LINK_LOCATION_OPTION, $default ) );
		return empty( $locations ) ? $default : $locations;
	}

	/**
	 * Optionally applies Tasty Links to the rendering of blocks.
	 *
	 * @param string $output Existing block output.
	 * @param array  $block  Block data.
	 * @return string
	 */
	public static function filter_render_block( $output, $block ) {
		global $wp;

		/**
		 * Allows Tasty Links to be optionally enabled for specific blocks.
		 *
		 * @var array $enabled_blocks Blocks to be enabled on.
		 */
		$enabled_blocks = apply_filters( 'tasty_links_enabled_rendered_blocks', array() );
		if ( ! empty( $block['blockName'] )
			&& in_array( $block['blockName'], $enabled_blocks, true ) ) {
			$skipped_links = array(
				get_permalink( get_the_ID() ),
			);
			if ( ! empty( $wp->request ) ) {
				$skipped_links[] = home_url( $wp->request );
			}

			$output = self::apply_links_to_content(
				$output,
				'frontend',
				null,
				$skipped_links
			);
		}

		return $output;
	}

	/**
	 * Appends the disclosure text to the content if it's not already there.
	 * Used by Tasty Recipes filter to add the disclosure text for links in a
	 * recipe.
	 *
	 * @param string $content Content to be processed.
	 * @return string
	 */
	public static function filter_the_content_late_for_integration( $content ) {
		if ( ! tasty_links_should_be_applied( 'tasty-recipes' ) ) {
			return $content;
		}
		$content = self::append_or_prepend_disclosure_to_content( $content );
		remove_filter( 'the_content', array( __CLASS__, 'filter_the_content_late_for_integration' ), 100 );
		return $content;
	}

	/**
	 * Indicate there's a post thumbnail when an alternate image URL is set.
	 *
	 * @param boolean $has_thumbnail Whether or not the post has a thumbnail.
	 * @param object  $post          Optional post object.
	 * @return boolean
	 */
	public static function filter_has_post_thumbnail_early( $has_thumbnail, $post ) {
		if ( $has_thumbnail ) {
			return $has_thumbnail;
		}
		$post = get_post( $post );
		if ( ! $post ) {
			return $has_thumbnail;
		}
		if ( Content_Model::POST_TYPE !== get_post_type( $post->ID ) ) {
			return $has_thumbnail;
		}
		return (bool) get_post_meta( $post->ID, Admin::IMAGE_URL_META_KEY, true );
	}

	/**
	 * Use the specified image URL if it's set.
	 *
	 * @param string  $html    Existing post thumbnail HTML.
	 * @param integer $post_id Post ID.
	 * @return string
	 */
	public static function filter_post_thumbnail_html_early( $html, $post_id ) {
		if ( Content_Model::POST_TYPE !== get_post_type( $post_id ) ) {
			return $html;
		}
		$image_url = get_post_meta( $post_id, Admin::IMAGE_URL_META_KEY, true );
		if ( empty( $html ) && ! empty( $image_url ) ) {

			$height = get_post_meta( $post_id, Admin::IMAGE_URL_HEIGHT_META_KEY, true );
			$width  = get_post_meta( $post_id, Admin::IMAGE_URL_WIDTH_META_KEY, true );
			if ( $height && $width ) {
				$height_width = sprintf( ' height="%d" width="%d"', $height, $width );
			} else {
				$height_width = '';
			}

			$html = sprintf(
				'<img src="%s"%s class="attachment-thumbnail tasty-links-image-url" data-pin-nopin="true" alt="%s" >',
				esc_url( $image_url ),
				$height_width,
				// translators: Alt text.
				esc_attr( sprintf( __( 'Image of %s', 'tasty-links' ), get_post_field( 'post_title', $post_id ) ) )
			);
		}
		return $html;
	}

	/**
	 * Apply links to the content.
	 *
	 * @param string          $content       Content to parse.
	 * @param string          $context       Context in which the links are applied.
	 * @param array|bool|null $atts          Whether or not the disclosure should be prepended if location='content'.
	 * @param array           $skipped_links Any links to skip when the links are applied.
	 * @return string
	 */
	public static function apply_links_to_content( $content, $context = 'frontend', $atts = array(), $skipped_links = array() ) {

		if ( ! in_array( $context, array( 'frontend', 'editor' ), true ) ) {
			return $content;
		}

		$link_posts = Content_Model::get_links();
		if ( empty( $link_posts ) ) {
			return $content;
		}

		self::prepare_content_atts( $atts );
		$disclosure_text     = get_option( Admin::DISCLOSURE_TEXT_OPTION, '' );
		$disclosure_location = get_option( Admin::DISCLOSURE_LOCATION_OPTION, 'link' );
		$should_prepend      = null;

		usort(
			$link_posts,
			function ( $a, $b ) {
				return strlen( $b->post_title ) - strlen( $a->post_title );
			}
		);

		$link_limit = get_option( Admin::MAX_LINK_OPTION, 30 );

		global $wpt_replace_count;
		/**
		 * Reset the global replace count for the block builder so the limit
		 * isn't used up on the recipe card. We need to be able to remove links
		 * in the admin area.
		 */
		if ( ! empty( $atts['reset_limit'] ) ) {
			$wpt_replace_count = array();
		}

		$is_block = function_exists( 'wp_is_block_theme' ) && wp_is_block_theme();
		foreach ( $link_posts as $link_post ) {
			$keyword = $link_post->post_title;
			$link    = get_post_meta( $link_post->ID, Admin::LINK_KEY, true );
			if ( empty( $link ) ) {
				continue;
			}

			if ( empty( $wpt_replace_count[ $keyword ] ) ) {
				$wpt_replace_count[ $keyword ] = 0;
			}

			if ( ! empty( $skipped_links ) ) {
				$should_skip = false;
				foreach ( $skipped_links as $skipped_link ) {
					$skipped_link_minus_protocol = str_replace( wp_parse_url( $skipped_link, PHP_URL_SCHEME ), '', $skipped_link );
					$link_minus_protocol         = str_replace( wp_parse_url( $link, PHP_URL_SCHEME ), '', $link );
					if ( rtrim( $link_minus_protocol, '/' ) === rtrim( $skipped_link_minus_protocol, '/' ) ) {
						$should_skip = true;
					}
				}
				if ( $should_skip ) {
					continue;
				}
			}
			if ( false === stripos( $content, $keyword ) ) {
				// Permit apostrophes to also match curled quotes.
				if ( false !== strpos( $keyword, "'" ) ) {
					if ( false === stripos( $content, str_replace( "'", '’', $keyword ) )
						&& false === stripos( $content, str_replace( "'", '&#8217;', $keyword ) ) ) {
						continue;
					}
					// Permit ampersands to also match ampersands.
				} elseif ( false !== strpos( $keyword, ' & ' ) ) {
					if ( false === stripos( $content, str_replace( ' & ', ' &amp; ', $keyword ) ) ) {
						continue;
					}
				} else {
					continue;
				}
			}
			if ( 'on' === get_post_meta( $link_post->ID, Admin::PREVENT_AUTO_LINKING_META_KEY, true ) ) {
				continue;
			}
			$rel_nofollow      = get_post_meta( $link_post->ID, Admin::REL_NOFOLLOW_META_KEY, true );
			$rel_sponsored     = get_post_meta( $link_post->ID, Admin::REL_SPONSORED_META_KEY, true );
			$open_new          = get_post_meta( $link_post->ID, Admin::OPEN_NEW_META_KEY, true );
			$append_disclosure = get_post_meta( $link_post->ID, Admin::APPEND_DISCLOSURE_META_KEY, true );
			$rel_value         = '';
			// rel="nofollow" is on by default.
			if ( 'off' !== $rel_nofollow ) {
				$rel_value .= ' nofollow';
			}
			// rel="sponsored" is off by default.
			if ( 'on' === $rel_sponsored ) {
				$rel_value .= ' sponsored';
			}
			// Add 'noopener' to external domains.
			if ( 'off' !== $open_new && wp_parse_url( home_url(), PHP_URL_HOST ) !== wp_parse_url( $link ) ) {
				$rel_value .= ' noopener';
			}
			if ( empty( $rel_value ) ) {
				$rel_value = 'off';
			}
			$front_boundary = '';
			if ( preg_match( '#^\w#', $keyword ) ) {
				$front_boundary = '\b';
			}
			$back_boundary = '';
			if ( preg_match( '#\w$#', $keyword ) ) {
				$back_boundary = '\b';
			}
			$quoted_keyword = preg_quote( $keyword, '/' );
			// Permit apostrophes to also match curled quotes.
			if ( false !== strpos( $quoted_keyword, "'" ) ) {
				$quoted_keyword = str_replace( "'", "(?:'|’|&\#8217;)", $quoted_keyword );
			}
			// Permit ampersands to also match ampersands.
			if ( false !== strpos( $quoted_keyword, ' & ' ) ) {
				$quoted_keyword = str_replace( ' & ', '\s(?:&|&amp;)\s', $quoted_keyword );
			}
			$pattern = "/<(a|span class=\"tasty-link\"|h[\d]|pre|script|style|iframe|code|applet|audio|canvas|button|textarea)(<(em|i|strong|b|span)>)?[^>]*>(?:[a-zA-Z0-9\s'\-\.,]|(?:<.*>.*<\/\1>))*(<\/(em|i|strong|b|span)>)?.*\n?<\/(a|span|h[\d]|pre|script|style|iframe|code|applet|audio|canvas|button|textarea)>(*SKIP)(*FAIL)|{$front_boundary}(" . $quoted_keyword . "){$back_boundary}(?=[^>]*(?:<|$))/isuU";
			if ( preg_match_all( $pattern, $content, $matches, PREG_OFFSET_CAPTURE ) ) {
				$diff = 0;
				foreach ( $matches[0] as $match ) {
					if ( $link_limit && $wpt_replace_count[ $keyword ] >= absint( $link_limit ) ) {
						continue;
					}
					$match[1] = $match[1] + $diff;
					$before   = substr( $content, 0, $match[1] );
					// Ignore content inbetween
					// <!-- tasty-links-ignore -->These apples<!-- /tasty-links-ignore -->.
					preg_match_all( self::IGNORE_CONTENT_REGEX, $before, $before_matches );
					if ( ! empty( $before_matches[0] ) ) {
						$last_before = array_pop( $before_matches[0] );
						// If the last marker wasn't a closing marker, then we're
						// inside the ignore part.
						if ( false === stripos( $last_before, '/tasty-links-ignore' ) ) {
							continue;
						}
					}

					$should_continue = false;
					$ignored_blocks  = array(
						'yoast/faq-block',
					);
					/**
					 * Allow specific blocks to be ignored.
					 *
					 * @var array $blocks Blocks to ignore.
					 */
					$ignored_blocks = apply_filters( 'tasty_links_ignored_blocks', $ignored_blocks, $context );

					foreach ( $ignored_blocks as $block ) {
						preg_match_all( '#/?wp:' . preg_quote( $block, '/' ) . '#', $before, $before_matches );
						if ( ! empty( $before_matches[0] ) ) {
							$last_before = array_pop( $before_matches[0] );
							// If the last marker wasn't a closing marker, then we're
							// inside the ignore part.
							if ( false === stripos( $last_before, '/wp:' . $block ) ) {
								$should_continue = true;
							}
						}
					}

					if ( $should_continue ) {
						continue;
					}

					if ( 'frontend' === $context ) {
						$replacement = '<a class="tasty-link" ';
						if ( 'off' !== $rel_value ) {
							$replacement .= 'rel="' . esc_attr( trim( $rel_value ) ) . '" ';
						}
						if ( 'off' !== $open_new ) {
							$replacement .= 'target="_blank" ';
						}
						if ( 'off' !== $append_disclosure ) {
							$replacement .= 'data-tasty-links-append-disclosure ';
						} else {
							$replacement .= 'data-tasty-links-no-disclosure ';
						}
						$replacement .= 'href="' . esc_url( $link ) . '">' . $match[0] . '</a>';
						if ( 'off' !== $append_disclosure
							&& $disclosure_text
							&& 'link' === $disclosure_location ) {
							$replacement .= ' <span class="tasty-links-disclaimer">' . $disclosure_text . '</span>';
						}
					} elseif ( 'editor' === $context ) {
						$replacement = '<span class="tasty-link" data-tasty-link-href="' . esc_url( $link ) . '">' . $match[0] . '</span>';
					}
					$use_limit = in_the_loop() || $is_block || ! empty( $atts['reset_limit'] ) || is_admin();
					if ( $use_limit ) {
						++$wpt_replace_count[ $keyword ];
					}

					$content = substr_replace( $content, $replacement, $match[1], strlen( $match[0] ) );
					$diff    = $diff + strlen( $replacement ) - strlen( $match[0] );
					if ( 'off' !== $append_disclosure && false !== $atts['prepend_disclosure'] ) {
						$should_prepend = true;
					}
					if ( 'off' === $append_disclosure && is_null( $should_prepend ) ) {
						$should_prepend = false;
					}
				}
			}
		}

		if ( $should_prepend
			&& 'frontend' === $context ) {
			$content = self::append_or_prepend_disclosure_to_content( $content );
		}

		if ( 'frontend' === $context ) {
			$content = preg_replace( self::IGNORE_CONTENT_REGEX, '', $content );
		}
		return $content;
	}

	/**
	 * Set the default values for passed parameters in apply_links_to_content().
	 *
	 * @since 1.3
	 * @param array|boolean|null $atts Parameters to use when applying links.
	 *
	 * @return void
	 */
	private static function prepare_content_atts( &$atts ) {
		$defaults = array(
			'prepend_disclosure' => true,
			'reset_limit'        => false,
		);
		if ( ! is_array( $atts ) ) {
			$atts = array(
				// Backwards compatibility. $atts used to be $prepend_disclosure.
				'prepend_disclosure' => $atts,
			);
		}
		$atts = array_merge( $defaults, $atts );
	}

	/**
	 * Appends or prepends the disclosure to the content, depending on config.
	 *
	 * @param string $content Existing content.
	 * @return string
	 */
	public static function append_or_prepend_disclosure_to_content( $content ) {
		$beginning_div = '<div class="tasty-links-general-disclaimer">';
		if ( false !== stripos( $content, $beginning_div ) ) {
			return $content;
		}
		$disclosure_location = get_option( Admin::DISCLOSURE_LOCATION_OPTION, 'link' );
		if ( 'content' !== $disclosure_location ) {
			return $content;
		}
		$disclosure_text = get_option( Admin::DISCLOSURE_TEXT_OPTION, '' );
		if ( ! $disclosure_text ) {
			return $content;
		}
		/**
		 * Whether or not the disclosure text appears before or after the content.
		 *
		 * @param string $location Either 'before' or 'after'.
		 */
		$location = apply_filters( 'tasty_links_disclosure_content_location', 'before' );
		if ( ! in_array( $location, array( 'before', 'after' ), true ) ) {
			$location = 'before';
		}
		$disclosure_text = '<div class="tasty-links-general-disclaimer">' . wpautop( $disclosure_text ) . '</div>';
		if ( 'before' === $location ) {
			$content = $disclosure_text . PHP_EOL . $content;
		} else {
			$content .= PHP_EOL . $disclosure_text;
		}
		return $content;
	}
}
