<?php
/**
 * Integration points with Tasty Recipes.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links\Integrations;

use Tasty_Links\Admin;
use Tasty_Links\Content_Model;
use Tasty_Links\Frontend;

/**
 * Integration points with Tasty Recipes.
 */
class Tasty_Recipes extends Base_Autolink {

	/**
	 * The setting name for this integration.
	 *
	 * @since 1.3
	 * @var string
	 */
	protected static $setting_name = 'tasty_recipe';

	/**
	 * Meta key used to store equipment ids.
	 *
	 * @var string
	 */
	const EQUIPMENT_IDS_META_KEY = 'tasty_links_ids';

	/**
	 * Renders the Equipment editor UI in the Tasty Recipes editor modal.
	 */
	public static function action_tasty_recipes_editor_after_video_url() {
		Admin::show_template_part( 'tasty-recipes/equipment-field' );
	}

	/**
	 * Adds Select2 and our custom JavaScript when Tasty Recipes is loaded.
	 */
	public static function action_tasty_recipes_enqueue_editor_scripts() {
		/*
		 * Select2
		 */
		$plugin_base = dirname( __DIR__ );
		wp_enqueue_script( 'tasty-links-select2', plugins_url( 'assets/lib/select2/select2.min.js', $plugin_base ), array( 'jquery' ), '4.0.6-rc.1', true );
		wp_enqueue_style( 'tasty-links-select2', plugins_url( 'assets/lib/select2/select2.min.css', $plugin_base ), null, '4.0.6-rc.1' );
		// Prevents conflict with other instances of Select2.
		wp_add_inline_script( 'tasty-links-select2', 'var existingSelect2 = jQuery.fn.select2 || null; if (existingSelect2) { delete jQuery.fn.select2; }', 'before' );
		wp_add_inline_script( 'tasty-links-select2', 'jQuery.fn.tastyLinksSelect2 = jQuery.fn.select2; if (existingSelect2) { delete jQuery.fn.select2; jQuery.fn.select2 = existingSelect2; }', 'after' );

		/*
		 * Internal
		 */
		$js_file = 'assets/js/tasty-recipes-editor.js';
		$fmtime  = filemtime( dirname( $plugin_base ) . '/' . $js_file );
		wp_enqueue_script(
			'tasty-links-tasty-recipes-editor',
			plugins_url( $js_file, $plugin_base ),
			array( 'jquery', 'jquery-ui-sortable', 'tasty-links-select2' ),
			$fmtime,
			true
		);
	}

	/**
	 * Strip out <span class="tasty-link"> from Tasty Recipe fields on save.
	 *
	 * @param mixed  $data Data to be saved.
	 * @param string $field Field to be saved.
	 * @return mixed
	 */
	public static function filter_tasty_recipes_pre_save_editor_data( $data, $field ) {
		switch ( $field ) {
			case 'description':
			case 'ingredients':
			case 'instructions':
			case 'notes':
				if ( false === stripos( $data, '<span class="tasty-link"' )
					&& false === stripos( $data, '<span class="tasty-link ' ) ) {
					break;
				}
				$data = preg_replace( '#<span([^>]+)?class="[^\"]*tasty-link[^\"]*"([^>]+)?>(.+)<\/span>#U', '$3', $data );
				break;
		}
		return $data;
	}

	/**
	 * Saves equipment IDs when Tasty Recipes saves a recipe.
	 *
	 * @param array  $data   Recipe data to save.
	 * @param object $recipe Recipe instance.
	 */
	public static function action_tasty_recipes_after_save_editor_data( $data, $recipe ) {
		if ( isset( $data['tasty-links-equipment-field'] ) ) {
			$value = ! empty( $data['tasty-links-equipment'] ) ? array_map( 'intval', (array) $data['tasty-links-equipment'] ) : array();
			update_post_meta( $recipe->get_id(), self::EQUIPMENT_IDS_META_KEY, $value );
		}
	}

	/**
	 * Filters recipe JSON to include our equipment IDs.
	 *
	 * @param array $recipe_json Existing recipe JSON data.
	 * @return array
	 */
	public static function filter_tasty_recipes_shortcode_response_recipe_json( $recipe_json ) {
		$rendered = array( 'description', 'ingredients', 'instructions', 'notes' );
		foreach ( $rendered as $field ) {
			$pass_atts             = self::set_link_reset( is_admin() );
			$recipe_json[ $field ] = Frontend::apply_links_to_content( $recipe_json[ $field ], 'editor', $pass_atts );
		}
		$recipe_json['tasty_links_equipment'] = get_post_meta( $recipe_json['id'], self::EQUIPMENT_IDS_META_KEY, true );
		return $recipe_json;
	}

	/**
	 * Add any links to the Tasty Recipes content.
	 * This is triggered when the recipe block is fetched while editing.
	 *
	 * @param string $content Content to be processed.
	 * @return string
	 */
	public static function filter_tasty_recipes_content( $content ) {
		if ( self::is_disabled() ) {
			return $content;
		}

		$skipped_links = self::skipped_links();

		$context = wp_is_json_request() ? 'editor' : 'frontend';
		$content = Frontend::apply_links_to_content( $content, $context, false, $skipped_links );

		// Check to see if one of the links needs the disclosure appended.
		if ( false !== stripos( $content, 'data-tasty-links-append-disclosure' )
			&& ! has_filter( 'the_content', array( 'Tasty_Links\Frontend', 'filter_the_content_late_for_integration' ), 100 ) ) {
			add_filter( 'the_content', array( 'Tasty_Links\Frontend', 'filter_the_content_late_for_integration' ), 100 );
		}

		return $content;
	}

	/**
	 * Filters Tasty Recipe card output to display equipment when present.
	 *
	 * @param array  $template_vars Existing template variables.
	 * @param object $recipe        Recipe object.
	 * @return array
	 */
	public static function filter_tasty_recipes_recipe_template_vars_early( $template_vars, $recipe ) {
		$equipment_ids                     = self::get_equipment_ids( $recipe );
		$template_vars['recipe_equipment'] = Content_Model::apply_disclosure_featured_links(
			Admin::get_template_part(
				'tasty-link-card',
				array(
					'equipment_ids' => $equipment_ids,
				)
			),
			$equipment_ids
		);
		if ( \Tasty_Links\Content_Model::should_disclosure_be_appended_or_prepended( $equipment_ids, 'equipment' )
			&& ! has_filter( 'the_content', array( 'Tasty_Links\Frontend', 'filter_the_content_late_for_integration' ), 100 ) ) {
			add_filter( 'the_content', array( 'Tasty_Links\Frontend', 'filter_the_content_late_for_integration' ), 100 );
		}
		return $template_vars;
	}

	/**
	 * Gets the equipment IDs for a recipe.
	 *
	 * @param object $recipe Recipe object.
	 * @return array
	 */
	public static function get_equipment_ids( $recipe ) {
		return get_post_meta( $recipe->get_id(), self::EQUIPMENT_IDS_META_KEY, true );
	}
}
